/* Whirl and Pinch, Earle F. Philhower III <earle@ziplabel.com>, March 12, 2003
 * 
 * The following routines were modified from Federico Mena-Quintero's
 * whirlpinch.c filter for The GIMP. 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Below is the original header from Frederico's and Scott's plugin:
 * orig: Whirl and Pinch plug-in --- two common distortions in one place
 * orig: Copyright (C) 1997 Federico Mena Quintero
 * orig: federico@nuclecu.unam.mx
 * orig: Copyright (C) 1997 Scott Goehring
 * orig: scott@poverty.bloomington.in.us
 */

// UI.CPP
// Basic user interface for a cdrLabel plug-in

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include "resource.h"
#include "plugin.h"

static HINSTANCE hInstance = NULL; // Our DLL instance
static MINIGRAPHIC *gfx = NULL;    // The MINIGRAPHIC we've been given
static POINT position = {0, 0};    // Where our dialog should be placed

// The actual computation engine.  Pass in preview to make it go faster
// but less accurate (i.e. only process 1/4 pixels or use integer instead
// of FP).
extern void ImageIt(int preview, int whirl, int pinch, int radius,
		    MINIGRAPHIC *src, MINIGRAPHIC *dst);

// Windows standard DLLMain, nothing to do except get hInstance
BOOL APIENTRY DllMain( HANDLE hModule, DWORD reason, LPVOID resvd )
{
  hInstance = (HINSTANCE)hModule;
	
  return TRUE;
}

// CenterDialog will place a dialog in the center of either its parent window
// or the screen if there is no parent
static void CenterDialog( HWND hwndDlg )
{
  HWND hwndOwner; 
  RECT rc, rcDlg, rcOwner; 

  hwndOwner = GetParent(hwndDlg);
  if (hwndOwner == NULL) hwndOwner = GetDesktopWindow(); 
  GetWindowRect(hwndOwner, &rcOwner); 
  GetWindowRect(hwndDlg, &rcDlg); 
  CopyRect(&rc, &rcOwner); 
  OffsetRect(&rcDlg, -rcDlg.left, -rcDlg.top); 
  OffsetRect(&rc, -rc.left, -rc.top); 
  OffsetRect(&rc, -rcDlg.right, -rcDlg.bottom); 
  SetWindowPos(hwndDlg, HWND_TOP, rcOwner.left + (rc.right / 2),
	       rcOwner.top + (rc.bottom / 2), 0, 0, SWP_NOSIZE); 
}

// PositionWindow moves a window to the specifiec point while checking to
// make sure it is still visible.  If the POINT == {0,0} then will center
static void PositionWindow( HWND hdlg, POINT *pt)
{
  HDC hdc;
  if (pt->x!=0 && pt->y!=0) {
    hdc=CreateIC("DISPLAY", "", "", NULL);
    pt->x = min( GetDeviceCaps(hdc, HORZRES)-100, pt->x);
    pt->x = max( pt->x, 0 );
    pt->y = min( GetDeviceCaps(hdc, VERTRES)-100, pt->y);
    pt->y = max( pt->y, 0 );
    DeleteDC(hdc);
    SetWindowPos(hdlg, HWND_TOP, pt->x, pt->y, 0,0, SWP_NOSIZE);
  } else
    CenterDialog(hdlg);
}

// Stores the dialog position in the POINT* passed in
static void SaveWindowPosition( HWND hdlg, POINT *pt )
{
  RECT saverect;
  GetWindowRect(hdlg, &saverect);
  pt->x = saverect.left;
  pt->y = saverect.top;
}


//---------------------------------------------------------------------------
// GraphicFilterInitialize takes care of any initialization and version
// checking, and returns TRUE if it is safe to continue.
EXPORT int GraphicFilterInitialize( MINIGRAPHICPARAM *param )
{
  DWORD type, len;

  // First, check that we can grok this plugin version!
  if (!param) return FALSE;
  if (param->version != PLUGINVERSION) return FALSE;

  // Safe to continue
  InitCommonControls(); // We use a slider bar...
  strncpy(param->title, "Whirl and Pinch", MAXTITLE);
  param->version = PLUGINVERSION;
  param->user = NULL;

  // Try and load the last saved dialog position from the registry
  type = REG_BINARY;
  len = sizeof(POINT);
  RegQueryValueEx(param->userKey, "Position", 0, &type,
		  (unsigned char *)&position, &len);
  
  return TRUE;
}

// About() is the license dialog box, we just load the GNU GPL into an edit
// box, set it to courier font, and wait for OK...
static BOOL CALLBACK About(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HRSRC hrsrc;
  HGLOBAL hglobal;

  switch(msg) {
  case WM_INITDIALOG:
    hrsrc = FindResource(hInstance, MAKEINTRESOURCE(IDR_LICENSE), "TEXT");
    hglobal = LoadResource(hInstance, hrsrc);
    SetDlgItemText(hDlg, IDC_EDIT, (LPCTSTR)hglobal);
    SendDlgItemMessage(hDlg, IDC_EDIT, WM_SETFONT,
		       (WPARAM)GetStockObject(ANSI_FIXED_FONT),
		       MAKELPARAM(TRUE, 0));
    return FALSE;

  case WM_COMMAND:
    if (LOWORD(wParam)==IDOK || LOWORD(wParam)==IDCANCEL)
      EndDialog(hDlg, TRUE);
    return TRUE;
  }
  return FALSE;
}

// Dialog is a standard DialogProc.  Its LPARAM is a pointer to the param
// structure passed in from cdrLabel.  On an update we do a preview, and
// on OK or Cancel we remove any temporary MINIGRAPHICS and return the
// final processed output.
static BOOL CALLBACK Dialog(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static MINIGRAPHICPARAM *param;
  static MINIGRAPHIC *mg;
  static int preview;

  int pinch, radius, whirl;

  switch(msg) {

  case WM_HSCROLL: // Sent by the thumbscrolls
    whirl = SendDlgItemMessage(hDlg, IDC_WHIRL, TBM_GETPOS, 0, 0);
    SetDlgItemInt(hDlg, IDC_WHIRLX, whirl, FALSE);
    pinch = SendDlgItemMessage(hDlg, IDC_PINCH, TBM_GETPOS, 0, 0);
    SetDlgItemInt(hDlg, IDC_PINCHX, pinch, FALSE);
    radius = SendDlgItemMessage(hDlg, IDC_RADIUS, TBM_GETPOS, 0, 0);
    SetDlgItemInt(hDlg, IDC_RADIUSX, radius, FALSE);
    // Calculate and draw it on-screen
    ImageIt( preview, whirl, pinch, radius, gfx, mg );
    param->PreviewMiniGraphic( mg );
    return TRUE;

  case WM_INITDIALOG: // Sent by Windoze
    param = (MINIGRAPHICPARAM*)lParam;  // The LPARAM is a MINIGRAPHICPARAM
    PositionWindow(hDlg, &position);    // Move dialog to last spot
    preview = TRUE;

    // MG will be the modified graphic
    mg = param->CreateMiniGraphic( gfx->width, gfx->height );

    SendDlgItemMessage(hDlg, IDC_WHIRL, TBM_SETRANGE, TRUE,
		       (LPARAM)MAKELONG(0, 100));
    SendDlgItemMessage(hDlg, IDC_WHIRL, TBM_SETPAGESIZE, 0, (LPARAM)10);
    SendDlgItemMessage(hDlg, IDC_WHIRL, TBM_SETPOS, TRUE, (LPARAM) 0);

    SendDlgItemMessage(hDlg, IDC_PINCH, TBM_SETRANGE, TRUE,
		       (LPARAM)MAKELONG(0, 100));
    SendDlgItemMessage(hDlg, IDC_PINCH, TBM_SETPAGESIZE, 0, (LPARAM)10);
    SendDlgItemMessage(hDlg, IDC_PINCH, TBM_SETPOS, TRUE, (LPARAM) 0);

    SendDlgItemMessage(hDlg, IDC_RADIUS, TBM_SETRANGE, TRUE,
		       (LPARAM)MAKELONG(0, 100));
    SendDlgItemMessage(hDlg, IDC_RADIUS, TBM_SETPAGESIZE, 0, (LPARAM)10);
    SendDlgItemMessage(hDlg, IDC_RADIUS, TBM_SETPOS, TRUE, (LPARAM) 50);

    SetFocus(GetDlgItem(hDlg, IDC_WHIRL));
    return FALSE;


  case WM_COMMAND: // Sent by a button
    switch (LOWORD(wParam)) {
    case IDC_ABOUT:
      DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_ABOUT),
		     GetParent(hDlg), About, NULL);
      return TRUE;

    case IDOK:
      // First, recalculate non-preview mode
      preview = FALSE;
      SendMessage(hDlg, WM_HSCROLL, 0, 0);
      SaveWindowPosition(hDlg, &position);  // Save dialog position 
      param->FreeMiniGraphic( gfx );        // Get rid of original graphic
      gfx = mg;                             // Return the modified one
      EndDialog(hDlg, TRUE);
      return TRUE;
		
    case IDCANCEL:
      SaveWindowPosition(hDlg, &position); // Save dialog position
      param->FreeMiniGraphic( mg );        // Don't want the modified graphic
      EndDialog(hDlg, FALSE);
      return TRUE;
		
    }
    break;
  }
  return FALSE;
}


// GraphicFilterUserInterface is called by cdrLabel whenever our menu item
// is selected.  We receive a graphic to work on and return the (possibly)
// modified result of our operation.
EXPORT MINIGRAPHIC *GraphicFilterUserInterface( MINIGRAPHICPARAM *param,
						HWND parent,
						MINIGRAPHIC *graphic )
{
  gfx = graphic; // May be modified by the DialogProg
  // Display our UI modally
  DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_DIALOG), parent,
		 Dialog, (LPARAM)param);
  // Return old or new graphic
  return gfx;
}



// GraphicFilterUninitialize is called when cdrLabel is shutting down
// We'd free any memory we had previously allocated and store any permanent
// options like dialog positions or dialog settings
EXPORT void GraphicFilterUninitialize( MINIGRAPHICPARAM *param )
{
  // Where should we stick the dialog box?
  RegSetValueEx(param->userKey, "position", 0, REG_BINARY,
		(unsigned char*)&position, sizeof(POINT));
}

